using System;
using System.Text;

using NUnit.Framework;

namespace Smdn.Protocols.Imap4 {
  [TestFixture]
  public class ImapEncodingTest {
    [Test]
    public void TestEncodeTransfer7BitValidChar()
    {
      ImapEncoding.Transfer7Bit.GetBytes("0004 append \"INBOX\" (\\Seen) {33}\r\n\x00\x20\x40\x60\x7f");
    }

    [Test]
    public void TestEncodeTransfer8BitValidChar()
    {
      ImapEncoding.Transfer8Bit.GetBytes("0004 append \"INBOX\" (\\Seen) {33}\r\n\x00\x20\x40\x60\x80\xa0\xc0\xe0\xff");
    }

    [Test]
    [ExpectedException(typeof(EncoderFallbackException))]
    public void TestEncodeTransfer7BitInvalidChar()
    {
      ImapEncoding.Transfer7Bit.GetBytes("\x20\x40\x60\x80");
    }

    [Test]
    [ExpectedException(typeof(EncoderFallbackException))]
    public void TestEncodeTransfer8BitInvalidChar()
    {
      ImapEncoding.Transfer8Bit.GetBytes("INBOX.日本語");
    }

    [Test]
    public void TestFromModifiedUTF7()
    {
      Assert.AreEqual("INBOX.日本語", ImapEncoding.FromModifiedUTF7String("INBOX.&ZeVnLIqe-"));

      Assert.AreEqual("&日&-本-&語-", ImapEncoding.FromModifiedUTF7String("&-&ZeU-&--&Zyw--&-&ip4--"));

      Assert.AreEqual("~peter/mail/台北/日本語", ImapEncoding.FromModifiedUTF7String("~peter/mail/&U,BTFw-/&ZeVnLIqe-"));

      Assert.AreEqual("☺!", ImapEncoding.FromModifiedUTF7String("&Jjo-!"), "☺");

      // padding: 0
      Assert.AreEqual("下書き", ImapEncoding.FromModifiedUTF7String("&Tgtm+DBN-"));
      // padding: 1
      Assert.AreEqual("サポート", ImapEncoding.FromModifiedUTF7String("&MLUw3TD8MMg-"));
      // padding: 2
      Assert.AreEqual("迷惑メール", ImapEncoding.FromModifiedUTF7String("&j,dg0TDhMPww6w-"));
    }

    [Test, ExpectedException(typeof(FormatException))]
    public void TestFromModifiedUTF7IncorrectForm()
    {
      ImapEncoding.FromModifiedUTF7String("&Tgtm+DBN-&");
    }

    [Test]
    public void TestFromModifiedUTF7Broken()
    {
      Assert.AreEqual("下書き", ImapEncoding.FromModifiedUTF7String("&Tgtm+DBN"));
      Assert.AreEqual("Tgtm+DBN-", ImapEncoding.FromModifiedUTF7String("Tgtm+DBN-"));
    }

    [Test]
    public void TestToModifiedUTF7()
    {
      Assert.AreEqual("INBOX.&ZeVnLIqe-", ImapEncoding.ToModifiedUTF7String("INBOX.日本語"));

      Assert.AreEqual("&-&ZeU-&--&Zyw--&-&ip4--", ImapEncoding.ToModifiedUTF7String("&日&-本-&語-"));

      Assert.AreEqual("~peter/mail/&U,BTFw-/&ZeVnLIqe-", ImapEncoding.ToModifiedUTF7String("~peter/mail/台北/日本語"));

      Assert.AreEqual("&Jjo-!", ImapEncoding.ToModifiedUTF7String("☺!"), "☺");

      // padding: 0
      Assert.AreEqual("&Tgtm+DBN-", ImapEncoding.ToModifiedUTF7String("下書き"));
      // padding: 1
      Assert.AreEqual("&MLUw3TD8MMg-", ImapEncoding.ToModifiedUTF7String("サポート"));
      // padding: 2
      Assert.AreEqual("&j,dg0TDhMPww6w-", ImapEncoding.ToModifiedUTF7String("迷惑メール"));
    }
  }
}
